***SUMMARY:  Replication file for IV results in Ballard-Rosa (2014) "Hungry for Change"
//Author:  Cameron Ballard-Rosa, drawing heavily from Scheve's "miest.ado"

clear all

//Set directory where datasets are stored here:
cd "C:\Users\cambr\Dropbox\CBR Data\Dissertation Empirics\" //UNC DESKTOP

//Change this local variable to specify which model you want (Models 1 & 2 are for Table 4, models 3-6 are for Table 8):
//FOR TABLE 4 (MAIN TEXT):
local model = 1 //IV regression -- baseline
*local model = 2 //IV regression -- full controls 
//FOR TABLE 8 (ONLINE APPENDIX):
*local model = 3 //IV regression, using only chicken, rice and soybean oil prices
*local model = 4 //IV regression (Newey 2 step), using prices X urbanization as instruments
*local model = 5 //IV regression (Newey 2 step), using prices as instruments (no year FE)
*local model = 6 //IV regression (Newey 2 step), using prices X urbanization as instruments (no year FE)


set more off

//Covariate specifications:
global baselineIV "wdi_urban ln_gdp debtgdp trade crisisyr ln_pop wdi_ase" 
global fullControlsIV "wdi_urban ln_gdp debtgdp trade crisisyr ln_pop oilrenpc foreign agereg wdi_ase"
global yearFE "yrdum2-yrdum49"
global countryFE "cdum2-cdum67"

//This matrix captures the F stat on excluded instruments from the first stage:
matrix define cdScores = (0)
matrix define rkScores = (0)
matrix define tempMat = (0)

//There are 5 imputed datasets, each of which is named "mi_ts#.dta", with the # replaced by 1-5:
local nsets = 5	

//Perform the analysis on each of the 5 multiply-imputed datasets:
local i = 1
while `i' <= `nsets' {
	use "mi_ts`i'.dta"
	sort ccode year
 	
//Basic specification:
	if `model' == 1 {
		global priceIVs "wheat_t25Xprice rice_t25Xprice chicken_t25Xprice sugar_t25Xprice soybeanoil_t25Xprice"
		ivprobit default_lead $baselineIV $countryFE $yearFE (foodimp = $priceIVs) if democ == 0, first vce(cluster ccode)
		ranktest (foodimp) ($priceIVs), wald partial($baselineIV $countryFE $yearFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($baselineIV $countryFE $yearFE)
		matrix rkScores = rkScores + r(chi2)
	}
//Full controls:
	if `model' == 2 {
		global priceIVs "wheat_t25Xprice rice_t25Xprice chicken_t25Xprice sugar_t25Xprice soybeanoil_t25Xprice"
		ivprobit default_lead $fullControlsIV $countryFE $yearFE (foodimp = $priceIVs) if democ == 0, first vce(cluster ccode)
		ranktest (foodimp) ($priceIVs), wald partial($fullControlsIV $countryFE $yearFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($fullControlsIV $countryFE $yearFE)
		matrix rkScores = rkScores + r(chi2)
	}
//Full controls, using only chicken, rice, and soybean oil as instruments:
	if `model' == 3 {
		global priceIVs "rice_t25Xprice chicken_t25Xprice soybeanoil_t25Xprice"
		ivprobit default_lead $fullControlsIV $countryFE $yearFE (foodimp = $priceIVs) if democ == 0, first vce(cluster ccode)
		ranktest (foodimp) ($priceIVs), wald partial($fullControlsIV $countryFE $yearFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($fullControlsIV $countryFE $yearFE)
		matrix rkScores = rkScores + r(chi2)
	}
//Newey two step, using only prices X urbanization as intruments (with year FE):
	if `model' == 4 {
		global priceIVs "wheatXurban riceXurban chickenXurban sugarXurban soybeanoilXurban"
		ivprobit default_lead $fullControlsIV $countryFE $yearFE (foodimp = $priceIVs) if democ == 0, twostep first vce(twostep)
		ranktest (foodimp) ($priceIVs), wald partial($fullControlsIV $countryFE $yearFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($fullControlsIV $countryFE $yearFE)
		matrix rkScores = rkScores + r(chi2)
	}
//Newey two step, using only prices as intruments (and no year FE):
	if `model' == 5 {
		global priceIVs "wheat_price rice_price sugar_price soybeanoil_price chicken_price"
		ivprobit default_lead $fullControlsIV $countryFE (foodimp = $priceIVs) if democ == 0, twostep first vce(twostep)
		ranktest (foodimp) ($priceIVs), wald partial($fullControlsIV $countryFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($fullControlsIV $countryFE)
		matrix rkScores = rkScores + r(chi2)
	}
//Newey two step, using price X urbanization as intruments (and no year FE):
	if `model' == 6 {
		global priceIVs "wheatXurban riceXurban chickenXurban sugarXurban soybeanoilXurban"
		ivprobit default_lead $fullControlsIV $countryFE (foodimp = $priceIVs) if democ == 0, twostep first vce(twostep)
		ranktest (foodimp) ($priceIVs), wald partial($fullControlsIV $countryFE)
		matrix cdScores = cdScores + r(chi2)
		ranktest (foodimp) ($priceIVs), wald cluster(ccode) partial($fullControlsIV $countryFE)
		matrix rkScores = rkScores + r(chi2)
	}
	
//Computation of multiple imputation estimates below:
	
	local nobs = e(N)
      local ngroups= e(N_g)
      local lgsize= e(g_max)
      local sgsize= e(g_min)
      local agsize= e(g_avg)
      local modeln= e(title)
      local grpvar= e(ivar)
      local timvar= e(tvar)
      local ecmd= e(cmd)
      local emodel= e(model)
      local family= e(family)
      local link= e(link)
      local corre= e(corr)
      local ccs= e(panels)
      local popt= e(vt)
      local censobs= e(N_cens)
      local uncobs= e(N) - e(N_cens)
      tempname bet Var b`i' V`i'

	matrix `bet' = e(b)
	matrix `Var' = e(V)
      
      matrix `bet' = `bet''
      local nueq = colsof(`bet')
      if `nueq' > 1 {
         matrix `b`i''=`bet'[.,1]
         local j = 2
         while `j' <= `nueq' {
            tempname t`j'
            matrix `t`j'' = `bet'[.,`j']
            matrix `b`i'' = `b`i'' \ `t`j''
            local j = `j' + 1
         }
      }         
      else {      
         matrix `b`i''= `bet'
      }
      matrix `V`i''= `Var'
      drop _all
      local i = `i' + 1
}

local i = 3
tempname qsum qbar usum ubar avgfc	
matrix `qsum' = `b1' + `b2'
matrix `usum' = `V1' + `V2'
while `i' <= `nsets' {
   matrix `qsum' = `qsum' + `b`i''
   matrix `usum' = `usum' + `V`i''
   local i = `i' + 1
}
scalar `avgfc' = 1/`nsets'
matrix `qbar' = `avgfc' * `qsum'
matrix `ubar' = `avgfc' * `usum'

local i = 1
while `i' <= `nsets' {
   tempname bdif`i' bdisq`i'
   matrix `bdif`i'' = `b`i'' - `qbar'
   matrix `bdisq`i'' = `bdif`i'' * `bdif`i'''
   local i = `i' + 1
}

local i = 3
tempname bimpv
matrix `bimpv' = `bdisq1' + `bdisq2'
while `i' <= `nsets' {
   matrix `bimpv' = `bimpv' + `bdisq`i''
   local i = `i' + 1
}

tempname bifc bifci bipv tvafc tvar abimv param
scalar `bifc' = `nsets' - 1
scalar `bifci' = 1 / `bifc'
matrix `bipv' = `bifci' * `bimpv'
scalar `tvafc' = 1 + `avgfc'
matrix `abimv' = `tvafc' * `bipv'
matrix `tvar' = `ubar' + `abimv'
scalar `param' = rowsof(`tvar')
local names : rownames(`tvar')
display 
display in g "Multiple Imputation Estimates"
display 
display in g "Model: `modeln'"
display in g "Group Variable (i): `grpvar'"
display in g "Dependent Variable: `dv'"
display 
display in g "Number of Observations: `nobs'"
display in g "Number of Groups: `ngroups'"
display in g "Obs per group: max = `lgsize'"
display in g "Obs per group: avg = `agsize'"
display in g "Obs per group: min = `sgsize'"
display in g _dup(63) "-" 
display in g  _skip(9) "|" _skip(6) "Coef." _skip(3) "Std. Err." /*
   */ _skip(7) "t"  _skip(10) "Df" _skip(5) "P>|t|"
display in g _dup(63) "-" 
local i = 1
while `i' <= `param' {
   tempname be`i' vare`i' see`i' te`i' pvalu`i' ubar`i' abimv`i' ve`i'
   scalar `be`i'' = `qbar'[`i',1]
   scalar `vare`i'' = `tvar'[`i',`i']
   scalar `ubar`i'' = `ubar'[`i',`i']
   scalar `abimv`i'' = `abimv'[`i',`i']
   scalar `see`i'' = sqrt(`vare`i'') 
   scalar `te`i'' = `be`i'' / `see`i''
   scalar `ve`i'' = `bifc' * (1 + (`ubar`i'' / `abimv`i''))^2
   scalar `pvalu`i'' = tprob(`ve`i'',`te`i'')
   local varna`i' : word `i' of `names'
   local sksp = 8 - length("`varna`i''")
   di in g _skip(`sksp') "`varna`i'' | " in y /*
           */ _col(13) %7.0g `be`i'' /*
    */ _col(24) %9.0g `see`i''/* 
    */ _col(35) %9.3f `te`i'' /*
           */ _col(46) %9.0f `ve`i'' /* 
           */ _col(55) %9.3f `pvalu`i''
   local i = `i' + 1
}

//This calculates the average Cragg-Donald F-stat for the first-stage excluded instruments:
matrix cdScores = cdScores / 5
matrix list cdScores
//This calculates the average rk (Kleibergen-Paap) stat for first-stage excluded instruments:
matrix rkScores = rkScores / 5
matrix list rkScores
